require "enet"

local play = {}
play.myTank = {}
play.tanks = {}
play.left, play.right = 0, 0
local ups = 60
local tbu = 1 / ups
local last

function play.load()
	play.tankBody = love.graphics.newImage("res/body.png")
	play.track = {love.graphics.newImage("res/track1.png"), love.graphics.newImage("res/track2.png"), love.graphics.newImage("res/track3.png")}
	play.width = love.graphics.getWidth()
	play.height = love.graphics.getHeight()

	love.keyboard.setKeyRepeat(true)
	love.graphics.setDefaultFilter("nearest", "nearest", 1)

	love.touch = love.touch or {}
	love.touch.getTouch = love.touch.getTouch or function() end
	love.touch.getTouchCount = love.touch.getTouchCount or function() return 0 end
end

function play.enter(data)
	play.net = {}
	play.net.host = enet.host_create()
	play.net.server = play.net.host:connect(data[1]..":9149")
	play.net.server:timeout(31, 5000, 5000)
	last = love.timer.getTime()
end

function play.update(dt)
	local event = play.net.host:service()	--get network events waiting
	while event do
		if event.type == "receive" then
			if play.myTank.x == nil then	--this should mean this is the first message received
				print("Received my data: "..event.data)
				local id, x, y, r, c1, c2, c3 = event.data:match("(%d+) (%d+) (%d+) (%d+) (%d+) (%d+) (%d+)")
				print("Interpreted as:")
				print("ID="..id.."  (x, y)=("..x..", "..y..")  rot="..r.."  color={"..c1..", "..c2..", "..c3.."}")
				play.myTank.id = tonumber(id)
				play.myTank.x=tonumber(x)
				play.myTank.y=tonumber(y)
				play.myTank.rot=tonumber(r)
				play.myTank.color={tonumber(c1), tonumber(c2), tonumber(c3)}
				play.myTank.lTrack, play.myTank.rTrack = 0, 0
			else 	--if not, this message should contain info about another tank
				--print(event.data)
				local id, x, y, r, lt, rt, c1, c2, c3 = event.data:match("([+-]?%d+) ([+-]?%d+) ([+-]?%d+) ([+-]?%d+) ([+-]?%d+) ([+-]?%d+) ([+-]?%d+) ([+-]?%d+) ([+-]?%d+)")
				if x == -3735928559 then
					table.remove(tanks, id)
				else
					--print("Updated tank "..id.."")
					play.tanks[id] = play.tanks[id] or {}
					play.tanks[id].id = id
					play.tanks[id].x = x / 1000
					play.tanks[id].y = y / 1000
					play.tanks[id].rot = r / 1000
					play.tanks[id].lTrack = lt
					play.tanks[id].rTrack = rt
					play.tanks[id].color = {c1, c2, c3}
				end
			end
		elseif event.type == "connect" then
			print(event.peer:connect_id() .. " connected.")
		elseif event.type == "disconnect" then
			print(event.peer:connect_id() .. " disconnected.")
		end
		event = play.net.host:service()	--any more events?
	end

	if play.myTank.x then
		local left, right = play.handleInput()
		play.handleMovement(left, right, dt)
	end

	if love.timer.getTime() - last > tbu and play.myTank.x then	--send data to clients every tbu seconds
		sendData()
		last = love.timer.getTime()
	end
end

function play.draw()
	for _, t in pairs(play.tanks) do
		if tonumber(t.id) ~= tonumber(play.myTank.id) then
			love.graphics.setColor(t.color)
			love.graphics.draw(play.track[math.floor(t.lTrack)%3+1], t.x, t.y, t.rot + math.pi / 2, 2, -2,  7, 8)
			love.graphics.draw(play.track[math.floor(t.rTrack)%3+1], t.x, t.y, t.rot + math.pi / 2, 2, -2, -3, 8)
			love.graphics.draw(play.tankBody, t.x, t.y, t.rot + math.pi / 2, 2, 2, 8, 8)
		end
	end

	if play.myTank.x then
		love.graphics.setColor(play.myTank.color)
		love.graphics.draw(play.track[math.floor(play.myTank.lTrack)%3+1], play.myTank.x, play.myTank.y, play.myTank.rot + math.pi / 2, 2, -2,  7, 8)
		love.graphics.draw(play.track[math.floor(play.myTank.rTrack)%3+1], play.myTank.x, play.myTank.y, play.myTank.rot + math.pi / 2, 2, -2, -3, 8)
		love.graphics.draw(play.tankBody, play.myTank.x, play.myTank.y, play.myTank.rot + math.pi / 2, 2, 2, 8, 8)
	end
end

function play.handleInput()
	local left, right = 0, 0
	if love.keyboard.isDown('up') then
		left = left + 0.5
		right = right + 0.5
	end
	if love.keyboard.isDown('left') then
		left = left + 0.5
	end
	if love.keyboard.isDown('right') then
		right = right + 0.5
	end

	for i = 1, love.touch.getTouchCount() do
		local index, x, y, pressure = love.touch.getTouch(i)
		left = left + (0.5 - x)
		right = right + (x - 0.5)
	end
	return left, right
end

function play.handleMovement(left, right, dt)
	local rotSpd = 5
	local movSpd = 100
	local trackSpd = 15

	local dr = right * dt * rotSpd - left  * dt * rotSpd
	local dp = left  * dt * movSpd + right * dt * movSpd

	play.myTank.rot = play.myTank.rot + dr
	play.myTank.x = play.myTank.x + math.cos(play.myTank.rot) * dp
	play.myTank.y = play.myTank.y + math.sin(play.myTank.rot) * dp

	play.myTank.lTrack = play.myTank.lTrack + trackSpd * dt * right
	play.myTank.rTrack = play.myTank.rTrack + trackSpd * dt * left
end

function sendData()
	play.net.server:send(play.myTank.id .." ".. math.floor(play.myTank.x*1000) .." ".. math.floor(play.myTank.y*1000) .." ".. math.floor(play.myTank.rot*1000) .." ".. math.floor(play.myTank.lTrack) .." ".. math.floor(play.myTank.rTrack))
end

return play
